@@ -6,6 +6,9 @@ a temperature measuring or a device status.
 """
 import logging
 
+from uuid import UUID
+
+
 from pyhap.const import (
     HAP_PERMISSION_READ,
     HAP_REPR_DESC,
@@ -51,14 +54,17 @@ HAP_FORMAT_DEFAULTS = {
     HAP_FORMAT_TLV8: "",
 }
 
-HAP_FORMAT_NUMERICS = (
+HAP_FORMAT_NUMERICS = {
     HAP_FORMAT_INT,
     HAP_FORMAT_FLOAT,
     HAP_FORMAT_UINT8,
     HAP_FORMAT_UINT16,
     HAP_FORMAT_UINT32,
     HAP_FORMAT_UINT64,
-)
+}
+
+DEFAULT_MAX_LENGTH = 64
+ABSOLUTE_MAX_LENGTH = 256
 
 # ### HAP Units ###
 HAP_UNIT_ARC_DEGREE = "arcdegrees"
@@ -78,11 +84,34 @@ PROP_VALID_VALUES = "ValidValues"
 
 PROP_NUMERIC = (PROP_MAX_VALUE, PROP_MIN_VALUE, PROP_MIN_STEP, PROP_UNIT)
 
+CHAR_BUTTON_EVENT = UUID("00000126-0000-1000-8000-0026BB765291")
+CHAR_PROGRAMMABLE_SWITCH_EVENT = UUID("00000073-0000-1000-8000-0026BB765291")
+
+
+IMMEDIATE_NOTIFY = {
+    CHAR_BUTTON_EVENT,  # Button Event
+    CHAR_PROGRAMMABLE_SWITCH_EVENT,  # Programmable Switch Event
+}
+
+# Special case, Programmable Switch Event always have a null value
+ALWAYS_NULL = {
+    CHAR_PROGRAMMABLE_SWITCH_EVENT,  # Programmable Switch Event
+}
+
 
 class CharacteristicError(Exception):
     """Generic exception class for characteristic errors."""
 
 
+def _validate_properties(properties):
+    """Throw an exception on invalid properties."""
+    if (
+        HAP_REPR_MAX_LEN in properties
+        and properties[HAP_REPR_MAX_LEN] > ABSOLUTE_MAX_LENGTH
+    ):
+        raise ValueError(f"{HAP_REPR_MAX_LEN} may not exceed {ABSOLUTE_MAX_LENGTH}")
+
+
 class Characteristic:
     """Represents a HAP characteristic, the smallest unit of the smart home.
 
@@ -119,6 +148,7 @@ class Characteristic:
             ValidValues, etc.
         :type properties: dict
         """
+        _validate_properties(properties)
         self.broker = None
         self.display_name = display_name
         self.properties = properties
@@ -138,6 +168,9 @@ class Characteristic:
 
     def _get_default_value(self):
         """Return default value for format."""
+        if self.type_id in ALWAYS_NULL:
+            return None
+
         if self.properties.get(PROP_VALID_VALUES):
             return min(self.properties[PROP_VALID_VALUES].values())
 
@@ -164,7 +197,9 @@ class Characteristic:
                 logger.error(error_msg)
                 raise ValueError(error_msg)
         elif self.properties[PROP_FORMAT] == HAP_FORMAT_STRING:
-            value = str(value)[:256]
+            value = str(value)[
+                : self.properties.get(HAP_REPR_MAX_LEN, DEFAULT_MAX_LENGTH)
+            ]
         elif self.properties[PROP_FORMAT] == HAP_FORMAT_BOOL:
             value = bool(value)
         elif self.properties[PROP_FORMAT] in HAP_FORMAT_NUMERICS:
@@ -174,8 +209,13 @@ class Characteristic:
                 )
                 logger.error(error_msg)
                 raise ValueError(error_msg)
+            min_step = self.properties.get(PROP_MIN_STEP)
+            if min_step:
+                value = value - (value % min_step)
             value = min(self.properties.get(PROP_MAX_VALUE, value), value)
             value = max(self.properties.get(PROP_MIN_VALUE, value), value)
+            if self.properties[PROP_FORMAT] != HAP_FORMAT_FLOAT:
+                value = int(value)
         return value
 
     def override_properties(self, properties=None, valid_values=None):
@@ -193,11 +233,16 @@ class Characteristic:
             raise ValueError("No properties or valid_values specified to override.")
 
         if properties:
+            _validate_properties(properties)
             self.properties.update(properties)
 
         if valid_values:
             self.properties[PROP_VALID_VALUES] = valid_values
 
+        if self.type_id in ALWAYS_NULL:
+            self.value = None
+            return
+
         try:
             self.value = self.to_valid_value(self.value)
         except ValueError:
@@ -224,9 +269,12 @@ class Characteristic:
         """
         logger.debug("set_value: %s to %s", self.display_name, value)
         value = self.to_valid_value(value)
+        changed = self.value != value
         self.value = value
-        if should_notify and self.broker:
+        if changed and should_notify and self.broker:
             self.notify()
+        if self.type_id in ALWAYS_NULL:
+            self.value = None
 
     def client_update_value(self, value, sender_client_addr=None):
         """Called from broker for value change in Home app.
@@ -239,11 +287,15 @@ class Characteristic:
             value,
             sender_client_addr,
         )
+        changed = self.value != value
         self.value = value
-        self.notify(sender_client_addr)
+        if changed:
+            self.notify(sender_client_addr)
         if self.setter_callback:
             # pylint: disable=not-callable
             self.setter_callback(value)
+        if self.type_id in ALWAYS_NULL:
+            self.value = None
 
     def notify(self, sender_client_addr=None):
         """Notify clients about a value change. Sends the value.
@@ -251,7 +303,8 @@ class Characteristic:
         .. seealso:: accessory.publish
         .. seealso:: accessory_driver.publish
         """
-        self.broker.publish(self.value, self, sender_client_addr)
+        immediate = self.type_id in IMMEDIATE_NOTIFY
+        self.broker.publish(self.value, self, sender_client_addr, immediate)
 
     # pylint: disable=invalid-name
     def to_HAP(self):
@@ -288,8 +341,9 @@ class Characteristic:
                     self.properties[PROP_VALID_VALUES].values()
                 )
         elif self.properties[PROP_FORMAT] == HAP_FORMAT_STRING:
-            if len(value) > 64:
-                hap_rep[HAP_REPR_MAX_LEN] = min(len(value), 256)
+            max_length = self.properties.get(HAP_REPR_MAX_LEN, DEFAULT_MAX_LENGTH)
+            if max_length != DEFAULT_MAX_LENGTH:
+                hap_rep[HAP_REPR_MAX_LEN] = max_length
         if HAP_PERMISSION_READ in self.properties[PROP_PERMISSIONS]:
             hap_rep[HAP_REPR_VALUE] = value
 
