@@ -25,8 +25,7 @@ class Loader:
     .. seealso:: pyhap/resources/characteristics.json
     """
 
-    def __init__(self, path_char=CHARACTERISTICS_FILE,
-                 path_service=SERVICES_FILE):
+    def __init__(self, path_char=CHARACTERISTICS_FILE, path_service=SERVICES_FILE):
         """Initialize a new Loader instance."""
         self.char_types = self._read_file(path_char)
         self.serv_types = self._read_file(path_service)
@@ -34,24 +33,25 @@ class Loader:
     @staticmethod
     def _read_file(path):
         """Read file and return a dict."""
-        with open(path, 'r') as file:
+        with open(path, "r", encoding="utf8") as file:
             return json.load(file)
 
     def get_char(self, name):
         """Return new Characteristic object."""
         char_dict = self.char_types[name].copy()
-        if 'Format' not in char_dict or \
-            'Permissions' not in char_dict or \
-                'UUID' not in char_dict:
-            raise KeyError('Could not load char {}!'.format(name))
-        return Characteristic.from_dict(name, char_dict)
+        if (
+            "Format" not in char_dict
+            or "Permissions" not in char_dict
+            or "UUID" not in char_dict
+        ):
+            raise KeyError("Could not load char {}!".format(name))
+        return Characteristic.from_dict(name, char_dict, from_loader=True)
 
     def get_service(self, name):
         """Return new service object."""
         service_dict = self.serv_types[name].copy()
-        if 'RequiredCharacteristics' not in service_dict or \
-                'UUID' not in service_dict:
-            raise KeyError('Could not load service {}!'.format(name))
+        if "RequiredCharacteristics" not in service_dict or "UUID" not in service_dict:
+            raise KeyError("Could not load service {}!".format(name))
         return Service.from_dict(name, service_dict, self)
 
     @classmethod
@@ -73,27 +73,3 @@ def get_loader():
     if _loader is None:
         _loader = Loader()
     return _loader
-
-
-# pylint: disable=unused-argument
-def get_char_loader(desc_file=None):
-    """Get a CharacteristicLoader with characteristic descriptions in the given file.
-
-    .. deprecated:: 2.0
-       Use `get_loader` instead.
-    """
-    logger.warning(
-        "'get_char_loader' is deprecated. Use 'get_loader' instead.")
-    return get_loader()
-
-
-# pylint: disable=unused-argument
-def get_serv_loader(desc_file=None):
-    """Get a ServiceLoader with service descriptions in the given file.
-
-    .. deprecated:: 2.0
-       Use `get_loader` instead.
-    """
-    logger.warning(
-        "'get_serv_loader' is deprecated. Use 'get_loader' instead.")
-    return get_loader()
